import { Observable, Observer, PartialObserver, Subscribable, Subscriber, Subscription, Unsubscribable } from "rxjs"
import { Bus, FisAppActor, ReceiverProfile, TransmisionMessage, TransmissionProfile, TransmitterProfile } from "./transport.interface"


export type TYPE = {
    adapterProfile: AdapterProfile,
    transmitterProfile: TransmitterProfile,
    receiverProfile: ReceiverProfile,
    requestResponseProfile: TransmissionProfile
}

export interface AdapterProfile {

}


export interface ConnectionManager {
    getTransmitterConnectionAdapter(): TransmitterConnectionAdapterBase
    getReceiverConnectionAdapter(): ReceiverConnectionAdapterBase
    getRequestResponseConnectionAdapter(): RequestResponseConnectionAdapter
}

export interface ConnectionAdaptorBase extends FisAppActor {
    connector: any
    connectorProfile: AdapterProfile | any
    connectionState: ConnectionState
    connectionStateBus: Observable<ConnectionState>
    adaptorTransmissionRole: AdaptorTransmissionRole

    subscribeConnectionState(): Observable<ConnectionState>
    publishConnectionState(): void
    connect(): void
    disconnect(): void
    send(message: any): Observable<any>
    emit(message: any): void
    emitStream(message: any): void
    subscribeMessages(messageFilter: any): Observable<any>
    getMessageBus(bus: Bus): Observable<any>
}


export interface TransmitterConnectionAdapterBase extends ConnectionAdaptorBase {

}

export interface ReceiverConnectionAdapterBase extends ConnectionAdaptorBase {

}

export interface RequestResponseConnectionAdapter extends TransmitterConnectionAdapterBase, ReceiverConnectionAdapterBase {

}

export type ConnectionState = 'ONLINE' | 'OFFLINE'
export enum AdaptorTransmissionRole {
    Transmitter,
    Receiver,
    RequestResponse
}

export enum Transport {
    Websocket,
    Grpc,
    Http
}
export interface TransportMessage {
    id: string,
    dateCreated: Date,
    transport: Transport,
    adapter: string,
    payload: TransmisionMessage
}