import { Observable, Subject } from "rxjs";
import { AdaptorTransmissionRole, RequestResponseConnectionAdapter } from "./connector.interface";
import { MessageTransmissionTransmitter } from "../transmission/msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "../transmission/msg.transmission.receiver";
import { MessageTransmissionRequestResponse } from "../transmission/msg.transmission.request-response";
import { ReceiverConnectionAdapter } from "../connector/connector.receiver";
import { TransmitterConnectionAdapter } from "../connector/connector.transmitter";

export interface MessageTransmissionManager {
    // what the hell is this here for
    getTransmissionInstance(): TransmissionProfile
}

export interface TransmissionProfile {
    id: string,
    receiverId: string,
    transmitterId: string,
    transmitter: MessageTransmissionTransmitter,
    receiver: MessageTransmissionReceiver,
    requestResponse: MessageRequestResponse
}
export interface FisAppActor {
    incomingMessageBus: Subject<any>
    outgoingMessageBus: Subject<any>

    send(message: any): Observable<any>
    emit(message: any): void
    emitStream(message: any): void
    subscribeMessages(messageFilter: any): Observable<any>
}

export interface MessageTransmissionBase extends FisAppActor {
    msgRepositoryService: any
    transmissionRole: AdaptorTransmissionRole
    adaptorsArray: any[]
    transmissionService: any // not sure what this is for, i guess they are interchangeable with adapterService
    adapterService: any // just use adapter Service, since it's grammatically closer to connectionadapter

    getMessageBus(bus: Bus): Observable<any>
}

export interface MessageReceiver extends MessageTransmissionBase {
    receiverProfile: ReceiverProfile

    setReceiver(receiverProfile: ReceiverProfile, role: AdaptorTransmissionRole, adapter: ReceiverConnectionAdapter): void
}

export interface MessageTransmitter extends MessageTransmissionBase {
    transmitterProfile: TransmitterProfile

    setTransmitter(transmitterProfile: TransmitterProfile, role: AdaptorTransmissionRole, adapter: TransmitterConnectionAdapter): void
}

export interface MessageRequestResponse extends MessageTransmissionBase {
    transmitterInstance: MessageTransmissionTransmitter
    receiverInstance: MessageTransmissionReceiver

    setTransmissionProfile(transmissionInfo: MessageTransmissionTransmitter, receiverInfo: MessageTransmissionReceiver, role: AdaptorTransmissionRole, adapter: RequestResponseConnectionAdapter): void
}

export interface FisMessage {
    header: {
        messageID: string,
        messageName: `NotificationMessage` | `ResponseMessage` | `RequestMessage`
    },
    data: any
}


export interface TransmitterProfile {
    id: string,
    name: string,
    dateCreated: Date
}

export interface ReceiverProfile {
    id: string,
    name: string,
    dateCreated: Date,
}
export interface TransmisionMessage {
    transmitter?: TransmitterProfile,
    receiver?: ReceiverProfile,
    payload?: FisMessage
}

export enum Bus {
    IncomingMessageBus,
    OutgoingMessageBus,
    ErrorMessageBus,
    NotificationMessageBus
}