import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { ConnectionManager } from "../connector/connector.manager";
import { MessageTransmissionManager as MessageTransmissionManagerInterface, ReceiverProfile, TransmissionProfile, TransmitterProfile } from "../interface/transport.interface";
import { v4 as uuidv4 } from 'uuid'
import { AdaptorTransmissionRole } from "../interface/connector.interface";
import { MessageTransmissionRequestResponse } from "./msg.transmission.request-response";
/* FOr now, to fill the holes/gaps so to speak, the FIS actor will interact wtih Message Transmission Manager, without having to wory about
the underyling transport protocol as well as managing client control, when it's acting as a transmitter. */
export class MessageTransmissionManager implements MessageTransmissionManagerInterface {
    transmission: TransmissionProfile[] = []
    connectionManager!: ConnectionManager

    constructor() {
        // logic here
    }

    // but this function also needs to talk to connection adaptor manager to instantiate the necessary adapter.
    getTransmissionInstance(): TransmissionProfile {
        let transmitter = this.getTransmitter()
        let receiver = this.getReceiver()
        let transmission: TransmissionProfile = {
            id: uuidv4(),
            receiverId: uuidv4(),
            transmitterId: uuidv4(),
            transmitter: transmitter,
            receiver: receiver,
            requestResponse: new MessageTransmissionRequestResponse(transmitter, receiver, AdaptorTransmissionRole.RequestResponse, this.connectionManager.getRequestResponseConnectionAdapter())
        }
        this.instantiateConnectionManager(transmission) // start an adapter
        this.transmission.push(transmission)
        return transmission
    }

    private getTransmitter(): MessageTransmissionTransmitter {
        let transmitterProfile: TransmitterProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionTransmitter(transmitterProfile, AdaptorTransmissionRole.Transmitter, this.connectionManager.getTransmitterConnectionAdapter())
    }

    private getReceiver(): MessageTransmissionReceiver {
        let receiverProfile: ReceiverProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionReceiver(receiverProfile, AdaptorTransmissionRole.Receiver, this.connectionManager.getReceiverConnectionAdapter())
    }

    private instantiateConnectionManager(transmissionProfile: TransmissionProfile): void {
        this.connectionManager = new ConnectionManager(transmissionProfile)
    }

}